# Copyright 2022 Cochise Ruhulessin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import pathlib

import pytest

from ckms.core import fetch
from ckms.core import RemoteBlob


@pytest.fixture
def blob():
    return RemoteBlob(
        provider='local',
        path=pathlib.Path(__file__).parent.joinpath('test.txt')
    )


@pytest.mark.asyncio
async def test_get_content(blob: RemoteBlob):
    assert await fetch(blob) == b'Hello world!'
