# Copyright 2022 Cochise Ruhulessin
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from typing import Any
from typing import Awaitable

from .ciphertext import CipherText
from .ioperationperformer import IOperationPerformer
from .iprovider import IProvider


class Decrypter(IOperationPerformer):
    __module__: str = 'ckms.types'
    algorithm: str | None
    provider: IProvider

    def decrypt(
        self,
        ct: bytes | CipherText,
        **kwargs: Any
    ) -> bytes | Awaitable[bytes]:
        """Decrypt `ct` using the configured algorithm."""
        assert hasattr(self, 'algorithm')
        assert hasattr(self, 'provider')
        assert isinstance(self.provider, IProvider)
        assert self.algorithm is not None
        if not isinstance(ct, CipherText): # pragma: no cover
            ct = CipherText(buf=ct)
        return self.provider.decrypt(
            algorithm=self.algorithm,
            key=self,
            ciphertext=ct
        )
